@@ -6,18 +6,19 @@ import argparse
 
 class cmd_base(argparse.ArgumentParser):
     def __skip_into_plugin(self, plugin):
-        if self.plugin_dir not in sys.path:
-            sys.path.append(self.plugin_dir)
+        if self.plugins_dir not in sys.path:
+            sys.path.append(self.plugins_dir)
         plugin_mod = f'{plugin}_ext'
-        plugin_py = os.path.join(self.plugin_dir, f'{plugin_mod}.py')
+        plugin_py = os.path.join(self.plugins_dir, f'{plugin_mod}.py')
+        plugin_name = plugin_py[:-7]
         assert os.path.exists(
             plugin_py), f'{os.path.realpath(plugin_py)} not exist!'
         sys.argv.remove(plugin)
+        sys.argv[0] = plugin_name
         ext = __import__(plugin_mod)
-        sys.exit(ext.cmd().main())
+        sys.exit(ext.cmd(prog=plugin).main())
 
     def __print_error(self, info):
-        print(info)
         sys.exit(255)
 
     def __get_help_info(self, args):
@@ -25,8 +26,8 @@ class cmd_base(argparse.ArgumentParser):
 
     def __get_names_from_opt_args(self, args):
         short_name, long_name = args['sname'], args['lname']
-        assert not (len(short_name) == 0 and len(long_name)
-                    == 0), 'both short and long name is empty!'
+        assert not (len(short_name) == 0 and len(long_name) ==
+                    0), 'both short and long name is empty!'
         if short_name and not short_name.startswith('-'):
             short_name = '-' + short_name
         if long_name and not long_name.startswith('--'):
@@ -100,18 +101,51 @@ class cmd_base(argparse.ArgumentParser):
     def __init_argv(self, opt_argv):
         for i in range(len(opt_argv)):
             if len(opt_argv[i]) == 5:
-                while opt_argv[i][0].startswith('-'):
-                    tmp = list(opt_argv[i])
+                tmp = list(opt_argv[i])
+                while tmp[0].startswith('-'):
                     tmp[0] = tmp[0][1:]
-                    opt_argv[i] = tuple(tmp)
-                while opt_argv[i][1].startswith('-'):
-                    tmp = list(opt_argv[i])
+                while tmp[1].startswith('-'):
                     tmp[1] = tmp[1][1:]
-                    opt_argv[i] = tuple(tmp)
+                tmp[0] = tmp[0].replace('-', '_')
+                tmp[1] = tmp[1].replace('-', '_')
+                opt_argv[i] = tuple(tmp)
 
-    def __init__(self, options_argv=[], enable_plugins=False, plugin_dir='cmds', **kwargs) -> None:
+    class CustomHelpFormatter(argparse.HelpFormatter):
+        def _get_builtin(self, key):
+            return __builtins__.get(key, None)
+
+        def _get_plugin_description(self) -> str:
+            path = self._get_builtin('plugins_dir')
+            if not path:
+                return ''
+            plugins_list = []
+            if os.path.exists(path):
+                for f in os.scandir(path):
+                    if f.name.endswith('_ext.py'):
+                        plugins_list.append(f.name[:-7])
+            if len(plugins_list) == 0:
+                return ''
+            tmp = "\n  ".join(plugins_list)
+            return f'\navailable plugins:\n  {tmp}\n'
+
+        def format_help(self) -> str:
+            text = super().format_help()
+            flag = '\noptions:'
+            pos = text.find(flag)
+            return ''.join([text[:pos], self._get_plugin_description(), text[pos:]])
+
+    def _set_builtin(self, key, value):
+        __builtins__[key] = value
+
+    def _unset_builtin(self, key):
+        if key in __builtins__:
+            del __builtins__[key]
+
+    def __init__(self, options_argv=[], enable_plugins=False, plugins_dir='cmds', prog=None, description=None, **kwargs) -> None:
         opt_argv = options_argv
-        super().__init__(prog='argparse_base')
+
+        super().__init__(prog=prog, description=description,
+                         formatter_class=self.CustomHelpFormatter)
 
         self.__init_argv(opt_argv)
         self.__cast_opt_argv(opt_argv)
@@ -128,16 +162,20 @@ class cmd_base(argparse.ArgumentParser):
                 **self.__get_kwargs_from_pos_args(args)
             )
 
-        if enable_plugins and len(sys.argv) > 1 and not sys.argv[1].startswith('-'):
-            self.plugin_dir = os.path.realpath(plugin_dir)
+        if enable_plugins:
+            self.plugins_dir = os.path.join(os.path.dirname(
+                os.path.realpath(sys.argv[0])), plugins_dir)
+            self._set_builtin('plugins_dir', self.plugins_dir)
             self.add_argument('plugin', default=None,
-                              nargs='?', help=os.path.join(plugin_dir, '<plugin>_ext.py'))
-            self.args = self.parse_args(args=sys.argv[1:2])
-            if getattr(self.args, 'plugin', None):
-                self.__skip_into_plugin(self.args.plugin)
-            assert False, 'impossible'
+                              nargs='?', help=os.path.join(self.plugins_dir, '<plugin>_ext.py'))
+            if len(sys.argv) > 1 and not sys.argv[1].startswith('-'):
+                self.args = self.parse_args(args=sys.argv[1:2])
+                if getattr(self.args, 'plugin', None):
+                    self.__skip_into_plugin(self.args.plugin)
+                assert False, 'impossible'
         else:
-            self.args = self.parse_args()
+            self._unset_builtin('plugins_dir')
+        self.args = self.parse_args()
 
     def getopt(self, opt):
         return self.get_opt(opt)
